import { objectSpread2 as _objectSpread2 } from '../_virtual/_rollupPluginBabelHelpers.js';
import batcher from 'atomic-batcher';
import axios from 'axios';
import { consoleTransport, logger } from 'react-native-logs';
import uuid from 'react-native-uuid';
import { tempMemoryTransport } from '../debug/temp-memory-transport.js';
import Config from './config.js';

/**
 * Firebase crashlytics transport
 */
var firebaseCrashlyticsTransport = function firebaseCrashlyticsTransport(crashlytics, _ref) {
  var msg = _ref.msg,
    level = _ref.level,
    rawMsg = _ref.rawMsg;
  crashlytics.log(msg);
  try {
    crashlytics.log(JSON.stringify(rawMsg));
  } catch (e) {
    // skip
  }
  if (level.text === 'error' && msg) {
    crashlytics.recordError(new Error(msg));
  }
};
/**
 * NewRelic transport
 */
var newRelicTransport = function newRelicTransport(NewRelic, _ref2) {
  var msg = _ref2.msg,
    level = _ref2.level,
    rawMsg = _ref2.rawMsg;
  try {
    NewRelic === null || NewRelic === void 0 || NewRelic.sendConsole('log', rawMsg);
  } catch (e) {
    // skip
  }
  if (level.text === 'error' && msg) {
    void (NewRelic === null || NewRelic === void 0 ? void 0 : NewRelic.recordError(new Error(msg)));
  }
};
/**
 * Grafana loki transport
 * Batching handle logs
 */
var grafanaLokiTransport = function grafanaLokiTransport(_ref3) {
  var appKeyName = _ref3.appKeyName,
    appBranch = _ref3.appBranch,
    grafana = _ref3.grafana,
    payload = _ref3.payload;
  return batcher(function (messages, cb) {
    var labels = {
      mobile: appKeyName,
      deployment: appBranch
    };
    // sort messages by level
    var streams = messages.reduce(function (res, _ref4) {
      var level = _ref4.level.text,
        msg = _ref4.msg,
        rawMsg = _ref4.rawMsg;
      if (!res[level]) {
        res[level] = {
          stream: _objectSpread2(_objectSpread2({}, labels), {}, {
            level: level,
            error: level === 'error'
          }),
          values: []
        };
      }
      var message;
      try {
        message = JSON.stringify(_objectSpread2({
          msg: msg,
          rawMsg: rawMsg
        }, payload));
      } catch (e) {
        message = "stringify error: ".concat(e.message);
      }
      res[level].values.push([String(Date.now() * 1000000), message]);
      return res;
    }, {});
    void axios.request({
      url: "".concat(grafana.url, "/loki/api/v1/push"),
      method: 'POST',
      headers: {
        Authorization: "Basic ".concat(grafana.token),
        'Content-Type': 'application/json'
      },
      data: {
        streams: Object.values(streams)
      }
    }).then(function () {
      return void cb();
    })["catch"](function (e) {
      console.error('Error send grafana logs:', Object.values(e.response));
      cb();
    });
  });
};
var initLogger = function initLogger() {
  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
    grafana = _ref5.grafana,
    crashlytics = _ref5.crashlytics,
    newrelic = _ref5.newrelic,
    _ref5$hasTempMemoryTr = _ref5.hasTempMemoryTransport,
    hasTempMemoryTransport = _ref5$hasTempMemoryTr === void 0 ? true : _ref5$hasTempMemoryTr,
    _ref5$params = _ref5.params,
    params = _ref5$params === void 0 ? {} : _ref5$params;
  /**
   * Log payload
   */
  var payload = {
    uuid: uuid.v4()
  };
  var setPayload = function setPayload(key, value) {
    payload[key] = value;
  };
  var appKeyName = Config.get('appKeyName');
  var appBranch = Config.get('appBranch');
  var isLocalDevelopment = Config.get('isLocalDevelopment');
  var isTests = Config.get('isTests');
  var grafanaTransport;
  if (grafana) {
    grafanaTransport = grafanaLokiTransport({
      appKeyName: appKeyName,
      appBranch: appBranch,
      grafana: grafana,
      payload: payload
    });
  }
  var config = _objectSpread2({
    enabled: !isTests,
    severity: isLocalDevelopment ? 'debug' : 'info',
    transport: function transport(props) {
      consoleTransport(props);
      if (!isLocalDevelopment) {
        // Production transport
        if (crashlytics) {
          firebaseCrashlyticsTransport(crashlytics, props);
        }
        if (newrelic) {
          newRelicTransport(newrelic, props);
        }
        if (grafanaTransport) {
          grafanaTransport(props);
        }
      }
      if (hasTempMemoryTransport) {
        tempMemoryTransport(props);
      }
    },
    transportOptions: isLocalDevelopment ? {
      colors: {
        info: 'blueBright',
        warn: 'yellowBright',
        error: 'redBright'
      }
    } : {
      colors: 'ansi'
    },
    levels: {
      trace: 0,
      debug: 1,
      info: 2,
      warn: 3,
      error: 4
    }
  }, params);
  var log = logger.createLogger(config);
  // extend log for adding some payload
  log.setPayload = setPayload;
  if (!Config.get('logger')) {
    Config.setup({
      logger: log
    });
  }
  return log;
};

export { initLogger as default };
//# sourceMappingURL=log.js.map
